﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using NJT.Comm;
using System.Collections;

namespace NJT.Sys
{
    public class LogItem
    {
        static public LogItem FromLine(DateTime dt, string strLine)
        {
            int iFirstSpaceIndex = strLine.IndexOf(' ');
            int iSecondSpaceIndex = strLine.IndexOf(' ', iFirstSpaceIndex + 1);
            if (iFirstSpaceIndex == -1 || iSecondSpaceIndex == -1)
                NjtCommException.Throw("Log行不合法.");
            LogItem item = new LogItem();
            string strLoggingTime = strLine.Substring(0, iFirstSpaceIndex).Trim();
            item.m_dtLogging = LogHelper.StringToDt(strLoggingTime, dt);
            item.m_strLogCode = strLine.Substring(iFirstSpaceIndex + 1, iSecondSpaceIndex - iFirstSpaceIndex - 1).Trim();
            item.m_strContent = strLine.Substring(iSecondSpaceIndex + 1);
            return item;
        }

        private string m_strLogCode;
        private string m_strContent;
        private DateTime m_dtLogging;
        public string LogCode { get { return m_strLogCode; } }
        public string Content { get { return m_strContent; } }
        public DateTime LogDateTime { get { return m_dtLogging; } }
    }

    /// <summary>
    /// 用于读取Log文件
    /// </summary>
    public class LogReader
    {
        protected class LogFileInfo
        {
            public DateTime m_dtYMD;
            public string m_strLogFileName;
            public int m_iLineNumber = 0;
        }

        const int m_iMinItemAmountPerPage = 10;
        const int m_iMaxItemAmountPerPage = 1000;

        public LogReader()
        {
        }

        public void Init(string strLogPath, string strLogType, DateTime dtStart, DateTime dtEnd)
        {
            if (!Verifier.IsLegalLoggingType(strLogType))
                NJT.Comm.NjtCommException.Throw(NJT.Comm.NjtExceptionCode.IncorrectArgument);

            if (strLogPath.LastIndexOf('\\') != strLogPath.Length - 1)
                strLogPath += "\\";
            if (!Directory.Exists(strLogPath))
                NJT.Comm.NjtCommException.Throw("路径不存在.");

            if (dtEnd<dtStart)
                NJT.Comm.NjtCommException.Throw("结束日期必须大于开始日期.");

            m_strLogPath = strLogPath;
            m_dtStart = new DateTime(dtStart.Year, dtStart.Month, dtStart.Day, 0, 0, 0, 0);
            m_dtEnd = new DateTime(dtEnd.Year, dtEnd.Month, dtEnd.Day, 0, 0, 0, 0);
            m_strLogType = strLogType;

            //尝试计算总条数
            m_arrLogFiles.Clear();
            DateTime dt = m_dtEnd;
            while (dt >= m_dtStart)
            {
                string strLogFile = LogHelper.GetLogFileName(m_strLogPath, m_strLogType, dt);
                if (File.Exists(strLogFile))
                {
                    LogFileInfo lfi = new LogFileInfo();
                    lfi.m_dtYMD = dt;
                    lfi.m_strLogFileName = strLogFile;
                    lfi.m_iLineNumber = GetLineNumberOfFile(strLogFile);
                    m_arrLogFiles.Add(lfi);
                }
                dt -= TimeSpan.FromDays(1);
            }

            //计算总条数
            m_iItemAmount = 0;
            foreach (LogFileInfo lfi in m_arrLogFiles)
            {
                m_iItemAmount += lfi.m_iLineNumber;
            }

            m_bInitialized = true;
        }

        public LogItem[] Read(PagingInfo pi)
        {
            if (!m_bInitialized)
                NJT.Comm.NjtCommException.Throw("未正确初始化.");
            if (pi.ItemNumberPerPage<m_iMinItemAmountPerPage || pi.ItemNumberPerPage>m_iMaxItemAmountPerPage)
                NJT.Comm.NjtCommException.Throw("每页显示条数必须在" + m_iMinItemAmountPerPage + "和" + m_iMaxItemAmountPerPage + "之间.");
            if (pi.RequestPage < 1)
                NJT.Comm.NjtCommException.Throw("请求页码必须大于等于1.");

            int iStartItemIndex = (pi.RequestPage - 1) * pi.ItemNumberPerPage;

            int iItemNumToReturn = m_iItemAmount-(pi.RequestPage-1)*pi.ItemNumberPerPage;
            if (iItemNumToReturn > pi.ItemNumberPerPage)
                iItemNumToReturn = pi.ItemNumberPerPage;
            if (iItemNumToReturn < 0)
                iItemNumToReturn = 0;

            LogItem[] itemsRtn = new LogItem[iItemNumToReturn];

            int iLeftItem = iItemNumToReturn;
            int iCountFromEnd = 0;
            int iCountFromEndPrev = 0;
            int i;
            for (i = 0; i < m_arrLogFiles.Count; i++ )
            {
                iCountFromEnd += ((LogFileInfo)m_arrLogFiles[i]).m_iLineNumber;
                if (iStartItemIndex<iCountFromEnd)
                    break;
                iCountFromEndPrev += ((LogFileInfo)m_arrLogFiles[i]).m_iLineNumber;
            }
            for (; i < m_arrLogFiles.Count; i++)
            {
                //看看应该从这个文件倒数第几行开始，倒读多少行
                LogFileInfo lfi = (LogFileInfo)m_arrLogFiles[i];
                int iStartIndexFromThisFilesEnd = iStartItemIndex - iCountFromEndPrev;
                int iItemLeftInThisFile = lfi.m_iLineNumber - iStartIndexFromThisFilesEnd;
                int iWantFromThisFile = iItemLeftInThisFile > iLeftItem ? iLeftItem : iItemLeftInThisFile;

                if (iWantFromThisFile > 0)
                {
                    int iReadEndIndex = lfi.m_iLineNumber - iStartIndexFromThisFilesEnd;
                    int iReadStartIndex = iReadEndIndex - iWantFromThisFile;

                    int iCurrLineNum = 0;
                    using (FileStream fs = File.Open(lfi.m_strLogFileName, FileMode.Open, FileAccess.Read, FileShare.ReadWrite))
                    {
                        using (StreamReader sr = new StreamReader(fs, Encoding.UTF8))
                        {
                            string strLine = null;
                            do
                            {
                                strLine = sr.ReadLine();
                                if (iCurrLineNum >= iReadStartIndex)
                                {
                                    int iIndexToSet = iItemNumToReturn - iLeftItem + iWantFromThisFile - (iCurrLineNum - iReadStartIndex) - 1;
                                    itemsRtn[iIndexToSet] = LogItem.FromLine(lfi.m_dtYMD, strLine);
                                }
                                iCurrLineNum++;
                                if (iCurrLineNum>=iReadEndIndex)
                                    break;
                            }
                            while (strLine != null);
                        }
                    }
                }
                iLeftItem -= iWantFromThisFile;
                iStartItemIndex += iWantFromThisFile;
                iCountFromEndPrev += lfi.m_iLineNumber;

                if (iLeftItem==0)
                    break;
            }
            
            if ((m_iItemAmount % pi.ItemNumberPerPage) == 0)
                pi.TotalPageNumber = m_iItemAmount / pi.ItemNumberPerPage;
            else
                pi.TotalPageNumber = 1 + m_iItemAmount / pi.ItemNumberPerPage;
            pi.TotalItemNumber = m_iItemAmount;

            return itemsRtn;
        }

        protected int GetLineNumberOfFile(string strFile)
        {
            int iLineNum = 0;
            using (FileStream fs = File.Open(strFile, FileMode.Open, FileAccess.Read, FileShare.ReadWrite))
            {
                using (StreamReader sr = new StreamReader(fs, Encoding.UTF8))
                {
                    while (null!=sr.ReadLine())
                    {
                        iLineNum++;
                    }
                }
            }
            return iLineNum;
        }

        public int TotalItemNumber
        {
            get
            {
                return m_iItemAmount;
            }
        }

        protected string m_strLogPath;
        protected string m_strLogType;
        protected DateTime m_dtStart;
        protected DateTime m_dtEnd;
        protected int m_iItemAmount;
        protected ArrayList m_arrLogFiles = new ArrayList();
        protected bool m_bInitialized = false;
    }
}
